# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grebble_flow/grpc/proto/processor_external.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='grebble_flow/grpc/proto/processor_external.proto',
  package='grebble_flow.grpc.proto',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n0grebble_flow/grpc/proto/processor_external.proto\x12\x17grebble_flow.grpc.proto\"K\n\x12\x46lowExecuteRequest\x12\x10\n\x08\x66lowName\x18\x01 \x01(\t\x12\x0f\n\x07\x63ontent\x18\x02 \x01(\t\x12\x12\n\nattributes\x18\x03 \x01(\t\"M\n\x13\x46lowExecuteResponse\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\t\x12\x12\n\nattributes\x18\x02 \x01(\t\x12\x11\n\tstreamEnd\x18\x03 \x01(\x08\x32}\n\x11\x45xternalProcessor\x12h\n\x07\x45xecute\x12+.grebble_flow.grpc.proto.FlowExecuteRequest\x1a,.grebble_flow.grpc.proto.FlowExecuteResponse\"\x00\x30\x01\x62\x06proto3'
)




_FLOWEXECUTEREQUEST = _descriptor.Descriptor(
  name='FlowExecuteRequest',
  full_name='grebble_flow.grpc.proto.FlowExecuteRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='flowName', full_name='grebble_flow.grpc.proto.FlowExecuteRequest.flowName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='content', full_name='grebble_flow.grpc.proto.FlowExecuteRequest.content', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='attributes', full_name='grebble_flow.grpc.proto.FlowExecuteRequest.attributes', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=77,
  serialized_end=152,
)


_FLOWEXECUTERESPONSE = _descriptor.Descriptor(
  name='FlowExecuteResponse',
  full_name='grebble_flow.grpc.proto.FlowExecuteResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='content', full_name='grebble_flow.grpc.proto.FlowExecuteResponse.content', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='attributes', full_name='grebble_flow.grpc.proto.FlowExecuteResponse.attributes', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='streamEnd', full_name='grebble_flow.grpc.proto.FlowExecuteResponse.streamEnd', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=154,
  serialized_end=231,
)

DESCRIPTOR.message_types_by_name['FlowExecuteRequest'] = _FLOWEXECUTEREQUEST
DESCRIPTOR.message_types_by_name['FlowExecuteResponse'] = _FLOWEXECUTERESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FlowExecuteRequest = _reflection.GeneratedProtocolMessageType('FlowExecuteRequest', (_message.Message,), {
  'DESCRIPTOR' : _FLOWEXECUTEREQUEST,
  '__module__' : 'grebble_flow.grpc.proto.processor_external_pb2'
  # @@protoc_insertion_point(class_scope:grebble_flow.grpc.proto.FlowExecuteRequest)
  })
_sym_db.RegisterMessage(FlowExecuteRequest)

FlowExecuteResponse = _reflection.GeneratedProtocolMessageType('FlowExecuteResponse', (_message.Message,), {
  'DESCRIPTOR' : _FLOWEXECUTERESPONSE,
  '__module__' : 'grebble_flow.grpc.proto.processor_external_pb2'
  # @@protoc_insertion_point(class_scope:grebble_flow.grpc.proto.FlowExecuteResponse)
  })
_sym_db.RegisterMessage(FlowExecuteResponse)



_EXTERNALPROCESSOR = _descriptor.ServiceDescriptor(
  name='ExternalProcessor',
  full_name='grebble_flow.grpc.proto.ExternalProcessor',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=233,
  serialized_end=358,
  methods=[
  _descriptor.MethodDescriptor(
    name='Execute',
    full_name='grebble_flow.grpc.proto.ExternalProcessor.Execute',
    index=0,
    containing_service=None,
    input_type=_FLOWEXECUTEREQUEST,
    output_type=_FLOWEXECUTERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_EXTERNALPROCESSOR)

DESCRIPTOR.services_by_name['ExternalProcessor'] = _EXTERNALPROCESSOR

# @@protoc_insertion_point(module_scope)



from concurrent import futures

import grpc
from grebble_flow.grpc.service.app import AppService

from grebble_flow.grpc.proto import processor_external_pb2_grpc, app_external_pb2_grpc
from grebble_flow.grpc.service.processor import ProcessorService


def start_server(port):
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    processor_external_pb2_grpc.add_ExternalProcessorServicer_to_server(
        ProcessorService(), server
    )
    app_external_pb2_grpc.add_ExternalAppServicer_to_server(AppService(), server)
    server.add_insecure_port(f"[::]:{port}")
    server.start()
    print("Server started")
    server.wait_for_termination()



from dataclasses_json import dataclass_json

from grebble_flow.managment.processor import find_all_processors


class FlowManager:
    def __init__(self):
        self.processor_classes = find_all_processors()

    def find_processor_instance_by_name(self, flow_name):
        result = [x for x in self.processor_classes if x.name == flow_name]
        if not result:
            raise Exception(f"Flow with name {flow_name} not found")
        if len(result) > 1:
            raise Exception(f"Multiple flow with name {flow_name}")

        return result[0]()

    def run(self, flow_name, content, attributes_req):
        flow_processor = self.find_processor_instance_by_name(flow_name)
        attributes = None
        if flow_processor.attributes:
            attributes = dataclass_json(flow_processor.attributes).from_dict(
                attributes_req
            )
        return flow_processor.execute(content=content, attributes=attributes)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grebble_flow/grpc/proto/app_external.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='grebble_flow/grpc/proto/app_external.proto',
  package='grebble_flow.grpc.proto',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n*grebble_flow/grpc/proto/app_external.proto\x12\x17grebble_flow.grpc.proto\"\x18\n\x16\x41ppExternalInfoRequest\">\n\x15ProcessorExternalInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x17\n\x0f\x61ttributeSchema\x18\x02 \x01(\t\"]\n\x17\x41ppInfoExternalResponse\x12\x42\n\nprocessors\x18\x01 \x03(\x0b\x32..grebble_flow.grpc.proto.ProcessorExternalInfo2}\n\x0b\x45xternalApp\x12n\n\x07\x41ppInfo\x12/.grebble_flow.grpc.proto.AppExternalInfoRequest\x1a\x30.grebble_flow.grpc.proto.AppInfoExternalResponse\"\x00\x62\x06proto3'
)




_APPEXTERNALINFOREQUEST = _descriptor.Descriptor(
  name='AppExternalInfoRequest',
  full_name='grebble_flow.grpc.proto.AppExternalInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=71,
  serialized_end=95,
)


_PROCESSOREXTERNALINFO = _descriptor.Descriptor(
  name='ProcessorExternalInfo',
  full_name='grebble_flow.grpc.proto.ProcessorExternalInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='grebble_flow.grpc.proto.ProcessorExternalInfo.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='attributeSchema', full_name='grebble_flow.grpc.proto.ProcessorExternalInfo.attributeSchema', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=97,
  serialized_end=159,
)


_APPINFOEXTERNALRESPONSE = _descriptor.Descriptor(
  name='AppInfoExternalResponse',
  full_name='grebble_flow.grpc.proto.AppInfoExternalResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='processors', full_name='grebble_flow.grpc.proto.AppInfoExternalResponse.processors', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=161,
  serialized_end=254,
)

_APPINFOEXTERNALRESPONSE.fields_by_name['processors'].message_type = _PROCESSOREXTERNALINFO
DESCRIPTOR.message_types_by_name['AppExternalInfoRequest'] = _APPEXTERNALINFOREQUEST
DESCRIPTOR.message_types_by_name['ProcessorExternalInfo'] = _PROCESSOREXTERNALINFO
DESCRIPTOR.message_types_by_name['AppInfoExternalResponse'] = _APPINFOEXTERNALRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

AppExternalInfoRequest = _reflection.GeneratedProtocolMessageType('AppExternalInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _APPEXTERNALINFOREQUEST,
  '__module__' : 'grebble_flow.grpc.proto.app_external_pb2'
  # @@protoc_insertion_point(class_scope:grebble_flow.grpc.proto.AppExternalInfoRequest)
  })
_sym_db.RegisterMessage(AppExternalInfoRequest)

ProcessorExternalInfo = _reflection.GeneratedProtocolMessageType('ProcessorExternalInfo', (_message.Message,), {
  'DESCRIPTOR' : _PROCESSOREXTERNALINFO,
  '__module__' : 'grebble_flow.grpc.proto.app_external_pb2'
  # @@protoc_insertion_point(class_scope:grebble_flow.grpc.proto.ProcessorExternalInfo)
  })
_sym_db.RegisterMessage(ProcessorExternalInfo)

AppInfoExternalResponse = _reflection.GeneratedProtocolMessageType('AppInfoExternalResponse', (_message.Message,), {
  'DESCRIPTOR' : _APPINFOEXTERNALRESPONSE,
  '__module__' : 'grebble_flow.grpc.proto.app_external_pb2'
  # @@protoc_insertion_point(class_scope:grebble_flow.grpc.proto.AppInfoExternalResponse)
  })
_sym_db.RegisterMessage(AppInfoExternalResponse)



_EXTERNALAPP = _descriptor.ServiceDescriptor(
  name='ExternalApp',
  full_name='grebble_flow.grpc.proto.ExternalApp',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=256,
  serialized_end=381,
  methods=[
  _descriptor.MethodDescriptor(
    name='AppInfo',
    full_name='grebble_flow.grpc.proto.ExternalApp.AppInfo',
    index=0,
    containing_service=None,
    input_type=_APPEXTERNALINFOREQUEST,
    output_type=_APPINFOEXTERNALRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_EXTERNALAPP)

DESCRIPTOR.services_by_name['ExternalApp'] = _EXTERNALAPP

# @@protoc_insertion_point(module_scope)



import json

from grebble_flow.grpc.proto import processor_external_pb2_grpc, processor_external_pb2
from grebble_flow.helpers.converter import grebble_converter
from grebble_flow.managment.manager import FlowManager


class ProcessorService(processor_external_pb2_grpc.ExternalProcessorServicer):
    def __init__(self, *args, **kwargs):
        self.flow_manager = FlowManager()

    def Execute(self, request, context):
        # get the string from the incoming request
        flow_name = request.flowName
        content = request.content
        attributes = request.attributes

        try:
            content = json.loads(content)
        except:
            pass
        try:
            attributes = json.loads(attributes)
        except:
            pass
        for item in self.flow_manager.run(flow_name, content, attributes):
            yield processor_external_pb2.FlowExecuteResponse(
                attributes=json.dumps(item.attributes, default=grebble_converter),
                content=json.dumps(item.content, default=grebble_converter),
                streamEnd=False,
            )

        yield processor_external_pb2.FlowExecuteResponse(
            data=None, attributes=None, streamEnd=True
        )
